﻿#NoEnv  																	; Recommended for performance and compatibility with future AutoHotkey releases.
; #Warn 																	; Enable warnings to assist with detecting common errors.
SendMode Input 																; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir% 												; Ensures a consistent starting directory.
;
; 	AHK_KaleidoscopeAssistant version 20240417 AutoHotkey Script
;		written by Ben Neece, 2024/04/17
;		Requires AutoHotkey (AHK) or compiled script as an executable
; 		written for AHK version 2, Kaleidoscope version 5.6.5c, on Windows 10
;
ahk_icon = %A_ScriptDir%\AHK_KaleidoscopeAssistant.ico
IfExist, %ahk_icon%
	Menu, Tray, Icon, %ahk_icon%
Menu, Tray, Tip, Kaleidoscope Assistant (SEBatHub)
global version = 20240417
global response = 															; global var for function
global responseDDL = 														; global var for function
global responseEdit = 														; global var for function


; functions
has_match(arr, rx) 															; function to check if array has a regex match
{
	for i, v in arr
		if (RegExMatch(v, rx))
			return True
	return False
}

count_match(arr, rxs) 														; function to check if array matches multiple regex
{
	cm = 0
	for i, rx in rxs
		if has_match(arr, rx)
			cm++
	return cm
}

has_val(arr, val) 															; function to check if any array value has a match
{
	for i, v in arr
		if (v == val)
			return True
	return False
}

within(arr, val) 															; function to check if any array value is within a string
{
	for i, v in arr
		if (InStr(val, v))
			return True
	return False
}

get_optional_value(name, desc, eg, fmt) 									; function to get optional typed message, with specific format
{
	response := " "
	While (response == " ")
	{		
		InputBox, response, Kaleidoscope Assistant (SEBatHub) - %name%, Type %desc% or leave it blank.%eg%
		if ErrorLevel
			Return
		response := Trim(response)
		if not response
			return response
		if (fmt == "string")
		{
			if RegExMatch(response, "^[^\w]{1}")
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! " name " must begin with an alphanumeric character or be left blank.`nPlease try again.", 2 			; notify user
				response := " "
			}
		}
		else if (fmt == "float")
		{
			if response is not float
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! " name " must be a decimal value.`nPlease try again.", 2
				response := " "
			}
		}
		else if (fmt == "integer")
		{
			if response is not integer
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! " name " must be an integer value.`nPlease try again.", 2
				response := " "
			}
		}
		else if (fmt == "number")
		{
			if response is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! " name " must be a numerical value.`nPlease try again.", 2
				response := " "
			}
		}
	}
	return response
}

get_optional_selection(name, desc, eg, opts) 								; function to get optional selection value
{
	response = 
	title := "Kaleidoscope Assistant (SEBatHub) - " name
	Gui, New,, %title%
	Gui, Add, Text,, Select %desc% or leave it blank.
	if (eg)
	{
		Gui, Add, Text,, %eg%
	}
	Gui, Add, DDL, w260 vresponse, %opts%
	Gui, Add, Button, gOK_WithinLoop, OK
	Gui, Show, w280
	WinWaitClose, %title%
	return response
}

get_optional_seltxt(name, desc, eg, opts) 									; function to get optional selection or typed value
{
	responseEdit = 
	title := "Kaleidoscope Assistant (SEBatHub) - " name
	Gui, New,, %title%
	Gui, Add, Text,, Select %desc%, type the value if not in the list, or leave it blank
	Gui, Add, DDL, w300 vresponseDDL, %opts%
	Gui, Add, Edit, w300 vresponseEdit
	Gui, Add, Button, gOK_WithinLoop, OK
	Gui, Show, w350
	WinWaitClose, %title%
	if (RegExMatch(Trim(responseEdit), "\w+"))
		return Trim(responseEdit)
	else if (responseDDL)
		return responseDDL
	else
		return
}

get_file_data(files_arr, files_type) 										; function to get SN, model, dates from binary data of files
{
	Out := {det_sn: [], dates: [], det_model: [], eight_three: ""}
	sn_catch := "file,serial"
	dm_catch := "file,model"
	det_matches := ["SD1 Mod1","Express","Swift","SD2"]
	for i, f in files_arr
	{
		GuiControl,, MainText, % "Checking " i " of " files_arr.MaxIndex() " files..."
		file_date = 
		file_det_sn = 
		file_det_model = 
		sn_offset = 6
		model_offset = 112
		; check for date in filename
		SplitPath, f, fname
		if RegExMatch(fname, ".*\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}.*") 	; new format filename w/ "_", extract
		{
			file_date := RegExReplace(fname, ".*(\d{4})-(\d{2})-(\d{2})_(\d{2})-(\d{2})-(\d{2}).*", "$1$2$3$4$5$6")
			Out[eight_three] := 0 											; store naming format
		}
		else if RegExMatch(fname, ".*\d{4}-\d{2}-\d{2} \d{2}-\d{2}-\d{2}.*") 	; new format filename w/ " ", extract
		{
			file_date := RegExReplace(fname, ".*(\d{4})-(\d{2})-(\d{2}) (\d{2})-(\d{2})-(\d{2}).*", "$1$2$3$4$5$6")
			Out[eight_three] := 0 											; store naming format
		}
		else if RegExMatch(fname, ".*\d{8}_\d{6}.(?i)wav") 						; SM4 format
		{
			file_date := RegExReplace(fname, ".*(\d{8})_(\d{6}).(?i)wav", "$1$2")
			Out[eight_three] := 0
		}
		else if RegExMatch(fname, "\w{8}[_\.]\d{2}", fname) 				; old format .**# or .zc filename, convert
		{
			file_date := 2000 + (Ord(SubStr(fname, 1, 1)) - Ord("A"))
			fnamePos2 := SubStr(fname, 2, 1)
			if fnamePos2 is integer
				file_date .= 0 . SubStr(fname, 2, 1)
			else
				file_date .= Ord(SubStr(fname, 2, 1)) - Ord("A") + 10
			file_date .= SubStr(fname, 3, 6) . SubStr(fname, 10, 2)
			Out[eight_three] := 1 											; store naming format
		}
		; notify if didn't find a date, otherwise add it to the list of dates
		if (RegExMatch(file_date, "\d{14}"))
			Out.dates.Push(file_date)
		else
			MsgBox,, Kaleidoscope Assistant (SEBatHub) - Notice!, % "Found no timestamp in`n" fname "`nContinuing in five seconds...", 5
		
		; read binary data to search for SN and model
		fdata := FileOpen(f, "r", "utf-8")
		if (files_type == ".wav") 											; quick & easy with some WAVs
		{
			fdata := fdata.Read()
			RegExMatch(fdata, "SM4BAT-FS", dm_check) 						; check if SM4 (need data from SM3 to add, but may be S3U)
			if dm_check
			{
				file_det_model := dm_check
				RegExMatch(fdata, "S4U\d{5}", sn_check)
				if sn_check
					file_det_sn := sn_check
			}
			else
			{
				RegExMatch(fdata, "Serial:(\w+)", sn_check)
				if sn_check
					file_det_sn := sn_check1
				RegExMatch(fdata, "Model:(\w+)", dm_check)
				if dm_check
					file_det_model := dm_check1
			}
		}
		else 																; more difficult with ZC
		{
			sn_offset := [6, 112, 396] 											; check specific offsets for 8-char SN, or 6-char w/o "SN"
			Loop % sn_offset.MaxIndex()
			{
				fdata.Seek(sn_offset[A_Index])
				fdata.RawRead(sn_check, 8)
				sn_check := StrGet(&sn_check, 8, "CP0")
				if (RegExMatch(sn_check, "[A-Z]{2}\d{6}|\d{6}.{2}")) || (sn_check == "NNNNNNNN")
				{
					if RegExMatch(sn_check, "\d{6}.{2}")
						sn_check := SubStr(sn_check,1,6)
					file_det_sn := sn_check
					break
				}
			}
			dm_offset := [70, 304, 336] 										; check specific offsets for 244 chars of text, look for known models
			Loop % dm_offset.MaxIndex()
			{
				fdata.Seek(dm_offset[A_Index])
				fdata.RawRead(dm_check, 244)
				dm_check := StrGet(&dm_check, 244, "CP0")
				if within(det_matches, dm_check)
				{
					if (InStr(dm_check, "Swift"))
						file_det_model := "Swift"
					else if (InStr(dm_check, "Express"))
						file_det_model := "Express"
					else if (InStr(dm_check, "SD1 Mod1"))
						file_det_model := "SD2"
					break
				}
			}
		}
		; add SN or None to list of SNs found
		if (file_det_sn == "")
		{
			file_det_sn := "None"
			;MsgBox,, Kaleidoscope Assistant (SEBatHub) - Notice!, % "Found no detector SN in`n" fname "`nFile will be skipped in this check, continuing in five seconds...", 5
		}
		if (Out.det_sn.MaxIndex() == "") & (file_det_sn != "")
			Out.det_sn.Push(file_det_sn)
		else if (Out.det_sn.MaxIndex() > 0) & (not has_val(Out.det_sn, file_det_sn))
			Out.det_sn.Push(file_det_sn)
		sn_catch .= "`n" . f . "," . file_det_sn 							; store file and SN in a list to export if multiple
		; add model or None to list of models found
		if (file_det_model == "")
		{
			file_det_model := "None"
			;MsgBox,, Kaleidoscope Assistant (SEBatHub) - Notice!, % "Found no detector model in`n" fname "`nFile will be skipped in this check, continuing in five seconds...", 5
		}
		if (Out.det_model.MaxIndex() == "") & (file_det_model != "")
			Out.det_model.Push(file_det_model)
		else if (Out.det_model.MaxIndex() > 0) & (not has_val(Out.det_model, file_det_model))
			Out.det_model.Push(file_det_model)
		dm_catch .= "`n" . f . "," . file_det_model 						; store file and model in a list to export if multiple
	}
	Out.dates := [RegExReplace(Min(Out.dates*), "\D") - 1, RegExReplace(Max(Out.dates*), "\D") + 1] 	; set to just Min and Max dates w/ 1 sec padding
	if (Out.det_sn.MaxIndex() > 1) 											; check if multiple found
		FileAppend, %sn_catch%, %sn_catch_name%, UTF-8 						; export CSV
	if (Out.det_model.MaxIndex() > 1) 										; check if multiple found
		FileAppend, %dm_catch%, %dm_catch_name%, UTF-8 						; export CSV
	Sleep, 1000
	Gui, Destroy
	; below to MsgBox is for testing
	; for k, v in Out.det_sn
	;	out_det_sn .= v
	; for k, v in Out.det_model
	;	out_det_model .= v
	; for k, v in Out.dates
	;	out_dates .= v . "`n"
	; MsgBox, % out_det_sn "`n" out_det_model "`n`n" out_dates
	return Out
}

list_files(dir) 	; function to return a string listing ZC files in a directory, also checks for SNs
{
	f := []
	Loop, Files, %dir%\*.*, FR
	{
		if (RegExMatch(A_LoopFileName, "\.zc$") || RegExMatch(A_LoopFileName, "\.\d{2}#") || RegExMatch(A_LoopFilename, "\.(WAV|wav)$"))
			f.Push(A_LoopFilePath)
	}
	return f
}

; beep and briefly introduce user to the script
SoundBeep, 840, 150
SoundBeep, 1050, 150
Gui, New,, Kaleidoscope Assistant (SEBatHub) - Welcome!
Gui, Add, Text,, Welcome to the Kaleidoscope Assistant (SEBatHub) (version %version%) for NABat!
Gui, Add, Text,, To process files with this script, open Kaleidoscope and press CTRL+F1`nTo restart the script, press CTRL+ALT+``
Gui, Add, Link,, Please ensure you have setup your cells and species lists in the two <a href="%A_ScriptDir%">CSV files</a> included with this script.`nCheck the <a href="%A_ScriptDir%\AHK_KaleidoscopeAssistant_README.txt">README file</a> for more information and detailed instructions.
Gui, Add, Link,, Visit the SE Bat Hub Forum <a href="https://www.sebathub.org/forum">here</a> if you need assistance or have suggestions related to this script.
Gui, Add, Link,, Official guidance on processing acoustic data for NABat is on their web site, <a href="https://www.nabatmonitoring.org/kaleidoscope-stationary">here</a>.
Gui, Add, Button, gOK_WithinLoop, OK
GuiControl, Focus, Button1
Gui, Show,, Kaleidoscope Assistant (SEBatHub) - Welcome!
Suspend, On
WinWaitClose, Kaleidoscope Assistant (SEBatHub) - Welcome!
Suspend, Off

^!`:: 																	; Hotkey to reload script
	Reload
return

^F1:: 																	; MAIN HOTKEY
	WinActivate, Kaleidoscope Pro 											; activate Kaleidoscope
	global script_runtime := A_Now 											; global timestamp for using throughout script
	; get Kaleidoscope version
	WinGetText, kalver, Kaleidoscope Pro 									; try an easy way first, through window text
	kalver := StrSplit(kalver, "`n")[2] 									; get Version text
	RegExMatch(kalver, "\d\.\d\.\d", kalver) 								; get only the version number
	if StrLen(kalver) == 0 													; if couldn't get version, try a harder way
	{
		WinGet, ProcessID, ID, A 												; get ahk_id
		WinGet, vPPath, ProcessPath, % "ahk_id " ProcessID 						; get process path
		fdata := FileOpen(vPPath, "r") 											; open the exe
		fdata.RawRead(BinData, FileLen := fdata.Length()) 						; read it as raw binary data
		kalver =
		Loop, 27 																; try to get the version
		{
			kalver .= Format("{:c}", NumGet(BinData, 2870847 + A_Index, "UChar"))
		}
		RegExMatch(kalver, "\d\.\d\.\d", kalver) 								; get only the version number
	}
	if StrLen(kalver) == 0 													; check another offset
	{
		Loop, 27
		{
			kalver .= Format("{:c}", NumGet(BinData, 1395375 + A_Index, "UChar"))
		}
		RegExMatch(kalver, "\d\.\d\.\d", kalver) 								; get only the version number
	}
	if StrLen(kalver) == 0 													; couldn't find it, ask user
	{
		InputBox, kalver, Kaleidoscope Assistant (SEBatHub) - Enter Kaleidoscope Version, Enter the Kaleidoscope software version.`ne.g.`, 5.5.0
	}
	ini_kalver := "version=" kalver 	 									; save version formatted for INI file
	;if (Format(SubStr(kalver, 1, 1) . SubStr(kalver, 3, 3), i) > 55) 		; v 5.6.0 added SMART tab but detection is not yet needed
	;	MsgBox yes
	kalver := "Kaleidoscope " kalver 										; combine with software name
	
	
	; Check if CSVs exist in path
	if !FileExist(A_ScriptDir "\AHK_KaleidoscopeAssistant_SppLists.csv")
	{
		MsgBox, 0, Kaleidoscope Assistant (SEBatHub) - ERROR!, ERROR!`nCould not find `nAHK_KaleidoscopeAssistant_SppLists.csv `nin path with this script.`n`nPlease download the file if you don't have it, modify it to your needs, and save it to the same path as this script.
		return
	}
	if !FileExist(A_ScriptDir "\AHK_KaleidoscopeAssistant_GRTSSppList.csv")
	{
		MsgBox, 0, Kaleidoscope Assistant (SEBatHub) - ERROR!, ERROR!`nCould not find `nAHK_KaleidoscopeAssistant_GRTSSppList.csv `nin path with this script.`n`nPlease download the file if you don't have it, modify it to your needs, and save it to the same path as this script.
		return
	}
	; read in species CSVs
	; SppLists csv first
	SppLists := [] 															; initialize array for values
	SppListsHeaders := [] 													; initialize array for headers
	Loop, read, %A_ScriptDir%\AHK_KaleidoscopeAssistant_SppLists.csv
	{
		if (A_Index == 1)													; headers
			SppListsHeaders := StrSplit(A_LoopReadLine,",")
		else 																; row of data
		{
			outer_index := A_Index 											; A_Index appears to reset w/ inner loop
			row_values := StrSplit(A_LoopReadLine,",") 						; split values to array
			for index, value in row_values 									; loop array
			{
				if (index == 1) 											; save SppList name as key, as well as SppList value
				{
					key := value
					SppLists[SppListsHeaders[1], outer_index - 1] := value
				}
				else 														; otherwise, save corresponding Spp code as 0/1 value
					SppLists[key, SppListsHeaders[index]] := value
			}
		}
	}
	SppListOptions = 														; initiate string to track unique lists
	for k, v in SppLists[SppListsHeaders[1]] 								; loop values
	{
		if (k == SppLists[SppListsHeaders[1]].MaxIndex()) 					; if last value, don't insert |
			SppListOptions .= v
		else 																; otherwise, insert |
			SppListOptions .= v . "|"
	}
	; GRTSSppList csv next
	GRTSSppLists := []														; initialize string to track unique lists
	GRTSList := [] 															; initialize array for GRTS matching to list
	Loop, read, %A_ScriptDir%\AHK_KaleidoscopeAssistant_GRTSSppList.csv
	{
		if (A_Index == 1)													; no need to track headers here
			continue			
		else 																; row of data
		{
			row_values := StrSplit(A_LoopReadLine,",") 						; split values to array
			if !(has_val(GRTSSppLists, row_values[2]))  					; store SppList if unique
				GRTSSppLists.Push(row_values[2])
			GRTSList[row_values[1]] := row_values[2] 						; store SppList for GRTS
		}
	}
	GRTSSppListOptions = 													; initiate str to store GRTS SppList options
	GRTSCSVExtras = 														; initiate str to store any extra SppList options in GRTS CSV
	for k, v in GRTSSppLists 												; loop values
	{
		if (k == GRTSSppLists.MaxIndex()) 									; if last value, don't insert |
			GRTSSppListOptions .= v
		else 																; otherwise, insert |
			GRTSSppListOptions .= v . "|"
		if !(has_val(SppLists[SppListsHeaders[1]], v)) 						; it value not in SppLists, store
		{
			if GRTSCSVExtras 												; if not first value, insert ,
				GRTSCSVExtras .= ", " . v
			else 															; otherwise, don't insert ,
				GRTSCSVExtras := v
		}
	}
	; compare SppLists btw CSVs
	SLCSVExtras = 															; initiate str to store any extra SppList options in SppLists CSV
	for k, v in SppLists[SppListsHeaders[1]] 								; loop values
	{
		if !(has_val(GRTSSppLists, v))
		{
			if SLCSVExtras
				SLCSVExtras .= ", " . v
			else
				SLCSVExtras := v
		}
	}
	if (SLCSVExtras or GRTSCSVExtras)
	{
		SoundBeep, 1050, 500
		error_msg = ERROR!`r`n`r`n
		if SLCSVExtras
		{
			error_msg .= "Found SppList(s) in SppList CSV that do not occur in GRTSSppList CSV:`r`n`t" SLCSVExtras
			if GRTSCSVExtras
				error_msg .= "`r`n`r`n"
		}
		if GRTSCSVExtras
			error_msg .= "Found SppList(s) in GRTSSppList CSV that do not occur in SppList CSV:`r`n`t" GRTSCSVExtras
		error_msg .= "`r`n`r`nClick OK to close this window, stop the script, and correct the error(s)."
		MsgBox, 0, Kaleidoscope Assistant (SEBatHub) - ERROR!, % error_msg
		return
	}
	
	
	; initial Kaleidoscope settings
	Control, Choose, 1, ComboBox1, Kaleidoscope Pro 						; bat mode
	ControlGet, Value, List, Selected, ComboBox2, Kaleidoscope Pro 			; get list of compute resources options
	Value := StrSplit(Value, "`n") 											; transform into array
	Value := Value.Length() 												; get max index of array - highest option
	Control, Choose, %Value%, ComboBox2, Kaleidoscope Pro 					; set to max compute resources
	
	; Batch tab
	ControlGet, Value, Tab,, _wx_SysTabCtl321, Kaleidoscope Pro 			; check if on Batch tab
	if Value != 1 															; change to batch tab if not on it
	{
		Value := Value - 1
		Control, TabLeft, %Value%, _wx_SysTabCtl321, Kaleidoscope Pro
		Sleep, 200
	}
	display_text := "Please type the path you wish to process. For proper NABat metadata attribution, it must only contain data from a single site or single mobile transect.`ne.g.,`nD:\Bat Data\2022\Raw\Site Name\Mobile 1"
	input_path = D:\\NoPathEntered
	While !(FileExist(input_path)) 											; get path from user
	{
		InputBox, input_path, Kaleidoscope Assistant (SEBatHub) - Enter Path, %display_text%
		if ErrorLevel
			Return
		files := list_files(input_path)
		if (files) 															; if files exist...
		{
			;if (has_match(files, ".*\.zc.*")) && (has_match(files, ".*\.\d{2}#.*")) 	; found both types
			if (count_match(files, [".*\.zc.*", ".*\.\d{2}#.*", ".\.(WAV|wav).*"] > 1)) 	; found multiple types
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub), % "Found multiple acoustic file types in this path, which will cause errors or not process all files.`nPlease check the data and enter a patch which contains just one acoustic data type.", 5
				input_path = D:\\NoPathEntered 								; reset and restart loop to ask again
				Continue
			}
			else if (has_match(files, ".*\.zc")) 							; found .zc
			{
				file_type = .zc
			}
			else if	(has_match(files, ".*\.\d{2}#.*"))						; found .**#
			{
				file_type = .**#
			}
			else if (has_match(files, ".*\.(WAV|wav).*"))
			{
				file_type = .wav
			}
			else
				file_type = non-bat
			nfiles := files.MaxIndex() 						; get number of files
		}
		else 																; if no files...
		{
			MsgBox,, Kaleidoscope Assistant (SEBatHub), % "No ZC files found in " input_path ".`nPlease enter another path", 4 	; notify
			input_path = D:\\NoPathEntered 									; reset and restart loop to ask again
		}
	}
	global sn_catch_name := input_path "\AHK_KaleidoscopeAssistant_FileError_MultipleSNs_" script_runtime ".csv" 	; global filename for use later, if needed
	global dm_catch_name := input_path "\AHK_KaleidoscopeAssistant_FileError_MultipleModels_" script_runtime ".csv" 	; global filename for use later, if needed
	Gui, New,, Kaleidoscope Assistant (SEBatHub) - Scanning files
	Gui, Add, Text, w280 vMainText, Found %nfiles% %file_type% files. Scanning files for metadata...
	Gui, Show, w300
	Sleep, 1000
	files_data := get_file_data(files, file_type)
	det_sn := files_data.det_sn
	det_model := files_data.det_model
	if files_data[eight_three] = 1
		zc_eightthree = 1 						; use 8.3 filenames
	else
		zc_eightthree = 0 						; don't use 8.3 filenames
	
	; start setting options
	Gui, New,, Kaleidoscope Assistant (SEBatHub) - Survey Type
	Gui, Add, Text,, Select the type of acoustic survey for this data
	Gui, Add, Radio, altsubmit gCheck vRadioGroup, Mobile Transect
	Gui, Add, Radio, altsubmit gCheck, Stationary Point
	Gui, Show
		Return
	Check:
		gui, submit, nohide
		if (RadioGroup == 1) {
			survey_form = NABat Metadata Form Kaleidoscope Mobile
			dur_min = 1
			Sleep, 1000
			Gui, Destroy
		}
		else if (RadioGroup == 2) {
			survey_form = NABat Metadata Form Kaleidoscope Stationary
			dur_min = 2
			Sleep, 1000
			Gui, Destroy
		}
	; check that the form exists, if not, link to download it and instructions (possible to automatically do it?)
	ControlGet, FormOptions, List,, ComboBox7, Kaleidoscope Pro 				; get list of metadata form options
	FormOptions := StrSplit(FormOptions, "`n") 									; transform into array
	if !(has_val(FormOptions, survey_form))
	{
		Gui, New,, Kaleidoscope Assistant (SEBatHub) - Error!
		if (InStr(survey_form, "Mobile"))
			Gui, Add, Text,, The NABat metadata form for mobile data is not an option. Did you install it?
		else
			Gui, Add, Text,, The NABat metadata form for stationary data is not an option. Did you install it?
		Gui, Add, Link,, Click <a href="https://www.sciencebase.gov/catalog/item/5dcdd2b5e4b069579760b15a">here</a> to go to the download page.
		Gui, Add, Text,, Next, click the project form drop-down menu on the Kaleidoscope window, click Add or Replace a Project Form, and select the form you downloaded.
		Gui, Add, Link,, Click <a href="https://www.nabatmonitoring.org/kaleidoscope-stationary">here</a> for instructions on adding it to Kaleidoscope.
		Gui, Add, Button, gCancel_FormOptions, Cancel
		Gui, Show
			Return
		Cancel_FormOptions:
			Gui, Destroy
			Return
	}

	display_text := "Please type the output path, e.g.,`nD:\Bat Data\2022\Raw\Site Name\Mobile 1 Kal`n`nImportant: This path should not contain any files or folders. The script will not check."
	InputBox, output_path, Kaleidoscope Assistant (SEBatHub) - Enter Path, %display_text%,,,,,,,,%input_path% Kal 	; insert the output path
	if ErrorLevel
		Return
	input_path := StrReplace(input_path, "\", "\\")
	output_path := StrReplace(output_path, "\", "\\")
	
	
	; start getting metadata
	grts_ID = 
	display_text := "Please type the GRTS Cell ID for this survey location."
	While !(grts_ID)
	{
		InputBox, grts_ID, Kaleidoscope Assistant (SEBatHub) - Enter GRTS Cell ID, %display_text%,,,130 	; get the GRTS ID
		if ErrorLevel
			Return
		grts_ID := Trim(grts_ID) 											; trim whitespace
		if grts_ID is not integer
		{
			MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! '" grts_ID "' is not an integer. Please type an integer.", 2 			; notify user
			grts_ID = 
		}
	}
	; try to get GRTS associated SppList based on CSV values
	spp_list =
	for k, v in GRTSList
		if (k == grts_ID)
			spp_list := GRTSList[grts_ID]
	if !(spp_list)
		spp_list := get_optional_seltxt("Species List", "the speclies list", "", ("|" + GRTSSppListOptions))
	if !(has_val(StrSplit(GRTSSppListOptions, "|"), spp_list))
	{
		SoundBeep, 1050, 500
		MsgBox, 1, Kaleidoscope Assistant (SEBatHub) - Notice!, % "Notice: '" spp_list "' was not found in your SppList CSVs!`nThe species list for auto-ID will default to " SppLists[SppListsHeaders[1], 1] ".`nYou will have to manually modify the selected species if needed.`n`nClick OK to continue or Cancel to restart."
		IfMsgBox Cancel
			Return
	}

	; get location name
	loc_name = 
	display_text =
	(LTrim Join`s
		Please type the Location Name for this data.
		`n`nUser-defined name for a single survey point or mobile transect. 
		Site Names for deployment locations could be 'NW', 'NE', 'SW,' and 
		'SE' or based on a nearby town, habitat type, property name, and so 
		forth. For a mobile transect, 'DT' (Driving Transect), is one 
		option. Consistency across years is essential.
	)
	While !(loc_name)
	{
		InputBox, loc_name, Kaleidoscope Assistant (SEBatHub) - Enter Location Name, %display_text%,,,250 	; get the location name, required
		if ErrorLevel
			Return
		loc_name := Trim(loc_name)
		if RegExMatch(loc_name, "^[^\w\(]{1}")
		{
			MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! '" loc_name "' is not alphanumeric or does not start with an accepted character.`nPlease type an acceptable name.", 2 			; notify user
			loc_name = 
		}
		; else if (StrLen(loc_name) < 2) || (StrLen(loc_name) > 6)
		; {
		; 	MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Location Name must be 2 - 6 characters.", 2 			; notify user
		; 	loc_name = 
		; }
	}
	cStart := "Choose" . files_data.dates[1] 								; set default values for start date chooser based on files
	cEnd := "Choose" . files_data.dates[2] 									; set default values for end date chooser based on files
	Gui, New,, Kaleidoscope Assistant (SEBatHub) - Select Survey Start and End
	Gui, Add, Text,, Format: YYYY-MM-DD HH:mm:ss
	Gui, Add, Text,, Select Survey Start Date and Time
	Gui, Add, DateTime, vStart %cStart%, yyyy-MM-dd HH:mm:ss
	Gui, Add, Text,, Select Survey End Date and Time
	Gui, Add, DateTime, vEnd %cEnd%, yyyy-MM-dd HH:mm:ss
	Gui, Add, Button, gOK_DateTimes, OK
	Gui, Show,,% chr(160)
		Return
	OK_DateTimes:
		Gui, Submit, Destroy
		FormatTime, Start, % Start, yyyy-MM-dd'T'HH:mm:ss
		FormatTime, End, % End, yyyy-MM-dd'T'HH:mm:ss
	
	; detector model check
	if (det_model.MaxIndex() > 1) 											; if found multiple det models, could be an error
	{
		SoundBeep, 1050, 500
		MsgBox, 4, Kaleidoscope Assistant (SEBatHub) - ERROR!, % "Error: Found " det_model.MaxIndex() " detector model names in the files!`nThis may be a serious data issue.`n`nPlease check that the path only contains data from one detector and/or survey event and re-run this script.`n`nReview the CSV with filenames and models found here: " dm_catch_name "`n`nDo you wish to ignore the potential error and process files as is?`nNote: you will be presented with a list of options for detector model if you continue."
		IfMsgBox Yes
			det_model = 
		IfMsgBox No
			Return
	}
	else 																	; array to value
		det_model := det_model[1]
	if (det_model == "SD2") 												; format model to metadata option
		det_model = TITLEY AnaBat SD2
	else if (det_model == "Express")
		det_model = TITLEY AnaBat Express
	else if (det_model == "Swift")
		det_model = TITLEY AnaBat Swift
	else if (det_model == "SM4BAT-FS")
		det_model = WILDLIFE ACOUSTICS SM4BAT-FS
	; else if (det_model == "None")
	;	det_model = 
	; ;;;;;;;;;;;;;;;;;;;;;;;;; COULD BE OPTIONAL, but prefer to ask
	else 																	; not detected or untested model, ask user to select one of the options
	{
		Gui, New,, Kaleidoscope Assistant (SEBatHub) - Detector Model
		Gui, Add, Text,, Select the acoustic detector model
		Gui, Add, DDL, w260 vdet_model, |AudioMoth 1.0.0|AudioMoth 1.1.0|AudioMoth 1.2.0|BINARY ACOUSTIC AR125|BINARY ACOUSTIC AR125-FG|BINARY ACOUSTIC AR180|BINARY ACOUSTIC AcroBat|BINARY ACOUSTIC iFR-IV|BINARY ACOUSTIC iFR-V|PETTERSSON D1000x|PETTERSSON D240x|PETTERSSON D500x|PETTERSSON M500|TITLEY AnaBat Express|TITLEY AnaBat SD1|TITLEY AnaBat SD2|TITLEY AnaBat Swift|TITLEY AnaBat Walkabout|WILDLIFE ACOUSTICS EM-Touch|WILDLIFE ACOUSTICS EM-Touch2|WILDLIFE ACOUSTICS EM-TouchPRO|WILDLIFE ACOUSTICS EM3/EM3+|WILDLIFE ACOUSTICS SM MICRO|WILDLIFE ACOUSTICS SM2|WILDLIFE ACOUSTICS SM2Bat+|WILDLIFE ACOUSTICS SM2Bat-192|WILDLIFE ACOUSTICS SM3Bat|WILDLIFE ACOUSTICS SM4BAT|WILDLIFE ACOUSTICS SM4BAT-FS|WILDLIFE ACOUSTICS SM4BAT-ZC|WILDLIFE ACOUSTICS SMMINI-BAT|WILDLIFE ACOUSTICS SMZC
		Gui, Add, Button, gOK_WithinLoop, OK
		Gui, Show, w280, gui_DetModel
		WinWaitClose, gui_DetModel
	}
	; detector SN check
	if (det_sn.MaxIndex() > 1)
	{
		SoundBeep, 1050, 500
		MsgBox, 4, Kaleidoscope Assistant (SEBatHub) - ERROR!, % "Error: Found " det_sn.MaxIndex() " detector serial numbers in the files!`nThis may be a serious data issue.`n`nPlease check that the path only contains data from one detector and/or survey event and re-run this script.`n`nReview the CSV with filenames and SNs found here: " sn_catch_name "`n`nDo you wish to ignore the potential error and process files as is?`nNote: detector SN will not be automatically filled in the metadata form if you continue."
		IfMsgBox Yes
			det_sn = 
		IfMsgBox No
			Return
	}
	else
		det_sn := det_sn[1]
	if (det_sn == "None")
		det_sn = 
	; ask user to set microphone model - not many options provided by NABat form and not sure what is accepted
	mic_model := get_optional_seltxt("Mic Model", "the mic model", "", "|Binary Acoustic Technology AR125|Binary Acoustic Technology AR150|Binary Acoustic Technology miniMIC|Pettersson D500x|Pettersson M500|TITLEY AnaBat Swift|Wildlife Acoustics SM3-U1|Wildlife Acoustics SMM-U1|Wildlife Acoustics SMM-U2|Wildlife Acoustics SMX-U1|Wildlife Acoustics SMX-US|Wildlife Acoustics SMX-UT|generic Directional|generic Extended Mic|generic Hi-Mic|generic Internal|generic Omni-directional|generic Regular") 	; get mic model
	mic_sn := get_optional_value("Mic SN", "the microphone serial number", "", "string")		; get mic SN
	mic_placeorien := " " 																		; get mic placement/orientation
	While (mic_placeorien == " ")
	{
		if (InStr(survey_form, "Mobile"))
		{
			InputBox, mic_placeorien, Kaleidoscope Assistant (SEBatHub) - Mic Placement, Type a brief description for the microphone placement during this survey or leave it blank
			if ErrorLevel
				Return
			if RegExMatch(mic_placeorien, "^[^\w]{1}")
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Mic Placement must begin with an alphanumeric character or be left blank.`nPlease try again.", 2
				mic_placeorien := " "
			}
		}
		else
		{
			Gui, New,, Kaleidoscope Assistant (SEBatHub) - Mic Orientation
			Gui, Add, Text,, Select the microphone orientation for this survey or leave it blank
			Gui, Add, DDL, w300 vmic_placeorien, |n|ne|e|se|s|sw|w|nw|vert
			Gui, Add, Button, gOK_WithinLoop, OK
			Gui, Show, w320, gui_orientation
			WinWaitClose, gui_orientation
		}
	}
	if (InStr(survey_form, "Stationary")) 															; many stationary-only variables
	{
		mic_height := get_optional_value("Mic Height", "the microphone height in meters", "", "number") 	; optionally get mic height
		mic_wp := get_optional_selection("Mic Weatherproofing", "whether you used weather proofing on the microphone", "", "|True|False") 	; optionally get mic weatherproofing
		lat = 
		lon = 
		While (!(lat) | !(lon))
		{
			Gui, New,, Kaleidoscope Assistant (SEBatHub) - Lat\, Long
			Gui, Add, Text,, Enter the latitude and longitude coordinates for this survey `nlocation in WGS84 decimal degrees.
			Gui, Add, Text,, `te.g., 34.67403228, -82.8351428
			Gui, Add, Edit, x10 y65 w150 vlat, %lat%
			Gui, Add, Edit, x+5 w150 vlon, %lon%
			Gui, Add, Button, x10 yp+25 gOK_WithinLoop, OK
			Gui, Show, w325, gui_latlon
			WinWaitClose, gui_latlon
			if lat is not float
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Latitude must be a decimal value.`nPlease try again.", 2
				lat = 
			}
			if lon is not float
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Longitude must be a decimal value.`nPlease try again.", 2
				lon = 
			}
		} ; ;;;;;;;; NOTE: COULD GET GRTS_ID WITH lat long PROGRAMATICALLY ;;;;;;;
		clutter_dist := get_optional_value("Distance to Clutter", "the distance to nearest clutter (e.g., vegetation, buildings, or other structure) in meters", "", "number") 	; optionally get clutter distance
		clutter_type := get_optional_selection("Clutter Type", "the type of the nearest clutter", "", "|Building|Rock|Vegetation|Water|Other") 	; optionally get clutter type
		clutter_perc := get_optional_value("Clutter Percent", "the percentage of clutter in the space surrounding the microphone", "", "integer") 	; optionally get clutter cover percentage
		water_dist := get_optional_value("Distance to Water", "the distance to nearest biologically significant water body in meters", "", "number") 	; optionally get water distance
		water_type := get_optional_value("Water Type", "the type of the nearest water body", "", "string") 	; optionally get nearest water body type
		habitat := get_optional_selection("Broad Habitat Type", "the broad habitat type of the site", "", "|agriculture|barren land|forest-conifer|forest-deciduous|forested wetland|grassland|shrubland|urban|water|wetland") 	; optionally get broad habitat type
		land_uc := get_optional_value("Land Unit Code", "the 4-letter abbreviation of the study area", "`ne.g., YELL for Yellowstone National Park, SOCA for South Carolina, 3LMA for Three Lakes WMA, etc.`nNote: multiple site names can occur within the same land unit code.", "string") 	; optionally get land unit code
		; ;;;;;;;; could do all of these on one gui, for stationary-only stuff ;;;;;;;;
	}
	
	contact := get_optional_value("Contact Info", "the name of the person or entity that conducted or organized the survey", "", "string")
	unusual := get_optional_value("Unusual Occurrences", "any occurrences which may impact the interpretation of the survey results", "`ne.g., low detector power or detector died, incorrect detector time or other settings, mic knocked down, equipment damaged, non-standard deployment, etc.", "string")
	
	; get weather variables on one gui window
	title = Kaleidoscope Assistant (SEBatHub) - Weather
	if (InStr(survey_form, "Stationary"))
		weather_desc = nightly average weather values
	else
		weather_desc = survey event weather values
	passing := False
	TempLow:=TempHigh:=RHLow:=RHHigh:=WindLow:=WindHigh:=CloudLow:=CloudHigh:=EventLow:=EventHigh:= ""
	while !(passing)
	{
		Gui, New,, %title%
		Gui, Add, Text,, Enter the %weather_desc% below. `nLeave all or any blank if needed.
		Gui, Add, Text, x10 y50 w100,
		Gui, Add, Text, x+5 w95 Center, Low
		Gui, Add, Text, x+5 w95 Center, High
		Gui, Add, Text, x10 y+4 w100 Right, Temp (C)
		Gui, Add, Edit, x+5 w95 vTempLow, %TempLow%
		Gui, Add, Edit, x+5 w95 vTempHigh, %TempHigh%
		Gui, Add, Text, x10 y+4 w100 Right, RH (`%)
		Gui, Add, Edit, x+5 w95 vRHLow, %RHLow%
		Gui, Add, Edit, x+5 w95 vRHHigh, %RHHigh%
		Gui, Add, Text, x10 y+4 w100 Right, Wind Speed (km/h)
		Gui, Add, Edit, x+5 w95 vWindLow, %WindLow%
		Gui, Add, Edit, x+5 w95 vWindHigh, %WindHigh%
		Gui, Add, Text, x10 y+4 w100 Right, Cloud Cover (`%)
		Gui, Add, Edit, x+5 w95 vCloudLow, %CloudLow%
		Gui, Add, Edit, x+5 w95 vCloudHigh, %CloudHigh%
		Gui, Add, Text, x10 y+4 w100 Right, Weather Event (str)
		Gui, Add, Edit, x+5 w95 vEventLow, %EventLow%
		Gui, Add, Edit, x+5 w95 vEventHigh, %EventHigh%
		Gui, Add, Button, x10 y+10 gOK_WithinLoop, OK
		Gui, Show, w325, %title%
		WinWaitClose, %title%
		passing := True
		if TempLow
		{
			if TempLow is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Low Temp must be a numerical value.`nPlease try again.", 2
				TempLow = 
				passing := False
			}
		}
		if TempHigh
		{
			if TempHigh is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! High Temp must be a numerical value.`nPlease try again.", 2
				TempHigh = 
				passing := False
			}
		}
		if RHLow
		{
			if RHLow is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Low RH must be a numerical value.`nPlease try again.", 2
				RHLow = 
				passing := False
			}
		}
		if RHHigh
		{
			if RHHigh is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! High RH must be a numerical value.`nPlease try again.", 2
				RHHigh = 
				passing := False
			}
		}
		if WindLow
		{
			if WindLow is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Low wind speed must be a numerical value.`nPlease try again.", 2
				WindLow = 
				passing := False
			}
		}
		if WindHigh
		{
			if WindHigh is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! High wind speed must be a numerical value.`nPlease try again.", 2
				WindHigh = 
				passing := False
			}
		}
		if CloudLow
		{
			if CloudLow is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Low cloud cover must be a numerical value.`nPlease try again.", 2
				CloudLow = 
				passing := False
			}
		}
		if CloudHigh
		{
			if CloudHigh is not number
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! High cloud cover must be a numerical value.`nPlease try again.", 2
				CloudHigh = 
				passing := False
			}
		}
		if EventLow
		{
			if EventLow is not alnum
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! Low weather event must be a string value.`nPlease try again.", 2
				EventLow = 
				passing := False
			}
		}
		if EventHigh
		{
			if EventHigh is not alnum
			{
				MsgBox,, Kaleidoscope Assistant (SEBatHub) - Error!, % "Error! High weather event must be a string value.`nPlease try again.", 2
				EventHigh = 
				passing := False
			}
		}
	}
	
	; check if Kaleidoscope version is in NABat metadata options, or get it from user
	software_options = Kaleidoscope 3.x|Kaleidoscope 4.0.0|Kaleidoscope 4.3.x|Kaleidoscope 4.5.0|Kaleidoscope 4.5.4|Kaleidoscope 4.5.5|Kaleidoscope 4.5.x|Kaleidoscope 4.x|Kaleidoscope 5.0.x|Kaleidoscope 5.1.x|Kaleidoscope 5.2.x|Kaleidoscope 5.3.x|Kaleidoscope 5.4.x|Kaleidoscope x|No Auto ID
	if (InStr(software_options, kalver))
	{
	}
	else if (InStr(software_options, SubStr(kalver, 1, -1) "x"))
	{
		kalver := SubStr(kalver, 1, -1) "x"
	}
	else if (InStr(kalver, "Kaleidoscope"))
	{
		kalver := "Kaleidoscope x"
	}
	else
	{
		Gui, New,, Kaleidoscope Assistant (SEBatHub) - Software Type
		Gui, Add, Text,, Select your Kaleidoscope software version
		Gui, Add, DDL, w260 vkalver, %software_options%
		Gui, Add, Button, gOK_WithinLoop, OK
		Gui, Show, w280, gui_kalver
		WinWaitClose, gui_kalver
	}

	manual := get_optional_value("Manual Vetter", "the name of the person or entity that will manually vet this data", "", "string") 	; get manual vetter
	
	; write Kaleidoscope options to ini, save, load ini
	ini_classifier = 
	(LTrim
		classifier=classifiers-Bats_of_North_America_5.4.0
		roc=2
	)
	; generate Spp inclusions based on SppList
	if !(has_val(StrSplit(GRTSSppListOptions, "|"), spp_list))
	{
		for k, v in SppLists[SppLists[SppListsHeaders[1], 1]]
		{
			ini_classifier .= "`n" . k . "=" . v
		}
	}
	else
	{
		for k, v in SppLists[spp_list]
		{
			ini_classifier .= "`n" . k . "=" . v
		}
	}
	; other settings based on defaults, data, and user selections
	ini_input = 
	(LTrim
		directory=%input_path%
		drivelabel=
		subdirs=1
		gpsfuzz=0
		metaform=%survey_form%
	)
	ini_input_wac := "enable=0"
	if (file_type == ".wav")
	{
		ini_input_wav = 
		(LTrim
			enable=1
			te=0
		)
		ini_input_zc := "enable=0"
	}
	else
	{
		ini_input_wav = 
		(LTrim
			enable=0
			te=0
		)
		ini_input_zc := "enable=1"
	}
	ini_output = 
	(LTrim
		directory=%output_path%
		drivelabel=
		subdirs=0
		channelsel=0
		max=
		noise=2
	)
	if (file_type == ".wav")
	{
		ini_output_wav =
		(LTrim
			enable0=1
			compress=0
			split=1
			te=1
		)
		ini_output_zc =
		(LTrim
			enable=0
			divratio=8
			83=0
			dotzc=1
		)
	}
	else
	{
		ini_output_wav = 
		(LTrim
			enable0=0
			compress=0
			split=1
			te=1
		)
		ini_output_zc = 
		(LTrim
			enable=1
			divratio=8
			83=%zc_eightthree%
			dotzc=1
		)
	}
	ini_output_gps = 
	(LTrim
		format=0
		waypoint=60
	)
	ini_analysis = 
	(LTrim
		freqmin0=8
		freqmax0=120
		durmin0=%dur_min%
		durmax0=50
		maxgap0=500
		cfmaxfreq=0
		cfmaxbw=0
		mincalls=3
		filterzc=1
		removedc=0
	)
	
	; write settings to INI and load it
	ini_name := "KaleidoscopeINI_" loc_name "_" script_runtime ".ini" 		; generate INI name
	FileAppend, %ini_kalver%`n[global]`nmode=0`n, %ini_name%, UTF-8 		; initiatlize INI file
	IniSections := ["ini_classifier", "ini_input", "ini_input_wac", "ini_input_wav", "ini_input_zc", "ini_output", "ini_output_wav", "ini_output_zc", "ini_output_gps", "ini_analysis"] 	; list of INI sections
	for key, Section in IniSections 										; loop through INI sections
	{
		; MsgBox % StrReplace(SubStr(Section, 5), "_", "/") " = `n" %Section%
		Value := %Section%
		Section := StrReplace(SubStr(Section, 5), "_", "/")
		IniWrite, %Value%, %ini_name%, %Section% 				; write them to the INI
		Sleep, 250
	}
	
	ini_path := StrReplace(A_ScriptFullPath, A_ScriptName, ini_name) 		; generate full path to INI
	WinActivate, Kaleidoscope Pro 											; activate Kaleidoscope
	Send, !f 																; open File menu
	Send, s 																; select Load Settings...
	WinWaitActive, Choose config .ini file 									; wait for file selection window to load
	Send, %ini_path% 														; send path to INI
	Sleep, 100 																; safety sleep
	Send, {Enter} 															; open the INI
	WinWaitActive, Kaleidoscope Pro 										; wait until Kaleidoscope is active
	Sleep, 100 																; safety sleep
	WinGetText, kprotext, Kaleidoscope Pro 									; get window text
	kprotext := Trim(StrSplit(kprotext, "`r`n")[5]) 						; get first line of text
	While (kprotext != "Bat Analysis Mode") 								; check to see if it's ready
	{
		Sleep, 100 															; sleep a bit
		WinGetText, kprotext, Kaleidoscope Pro 								; refresh window text
		kprotext := Trim(StrSplit(kprotext, "`r`n")[5]) 					; get first line
	}
	Sleep, 200
	; start setting metadata info
	ControlSetText, Edit4, %grts_ID%, Kaleidoscope Pro 						; set GRTS ID
	ControlSetText, Edit5, %loc_name%, Kaleidoscope Pro 					; set Location Name
	; for stationary, enter lat (Edit6) and long (Edit7)
	if (InStr(survey_form, "Mobile"))
		x = 0
	else
	{
		x = 2
		ControlSetText, Edit6, %lat%, Kaleidoscope Pro 							; set stationary latitude
		ControlSetText, Edit7, %lon%, Kaleidoscope Pro 							; set stationary longitude
	}
	ControlSetText, % "Edit" + (6 + x), %Start%, Kaleidoscope Pro 			; set Start Time
	ControlSetText, % "Edit" + (7 + x), %End%, Kaleidoscope Pro				; set End Time
	ControlSetText, % "Edit" + (8 + x), %det_model%, Kaleidoscope Pro 		; set det model
	ControlSetText, % "Edit" + (9 + x), %det_sn%, Kaleidoscope Pro			; set det SN if found
	ControlSetText, % "Edit" + (10 + x), %mic_model%, Kaleidoscope Pro 		; set microphone model
	ControlSetText, % "Edit" + (11 + x), %mic_sn%, Kaleidoscope Pro 		; set microphone SN
	ControlSetText, % "Edit" + (12 + x), %mic_placeorien%, Kaleidoscope Pro ; set mic placement/orientation
	if (InStr(survey_form, "Stationary"))
	{
		ControlSetText, % "Edit" + (13 + x), %mic_height%, Kaleidoscope Pro 	; set mic height
		ControlSetText, % "Edit" + (14 + x), %clutter_dist%, Kaleidoscope Pro 	; set dist to clutter
		ControlSetText, % "Edit" + (15 + x), %clutter_type%, Kaleidoscope Pro 	; set clutter type
		ControlSetText, % "Edit" + (16 + x), %water_dist%, Kaleidoscope Pro 	; set dist to water
		ControlSetText, % "Edit" + (17 + x), %water_type%, Kaleidoscope Pro 	; set water type
		ControlSetText, % "Edit" + (18 + x), %clutter_perc%, Kaleidoscope Pro 	; set percent clutter
		ControlSetText, % "Edit" + (19 + x), %habitat%, Kaleidoscope Pro 		; set broad habitat
		ControlSetText, % "Edit" + (20 + x), %land_uc%, Kaleidoscope Pro 		; set land unit code
		ControlSetText, % "Edit" + (22 + x), %mic_wp%, Kaleidoscope Pro 		; set weatherproofing
		x = 10
	}
	ControlSetText, % "Edit" + (13 + x), %contact%, Kaleidoscope Pro 		; set contact info
	ControlSetText, % "Edit" + (15 + x), %unusual%, Kaleidoscope Pro 		; set unusual occurrences
	ControlSetText, % "Edit" + (16 + x), %TempLow%, Kaleidoscope Pro 		; set low temp
	ControlSetText, % "Edit" + (17 + x), %TempHigh%, Kaleidoscope Pro 		; set high temp
	ControlSetText, % "Edit" + (18 + x), %RHLow%, Kaleidoscope Pro 			; set low RH %
	ControlSetText, % "Edit" + (19 + x), %RHHigh%, Kaleidoscope Pro 		; set high RH %
	ControlSetText, % "Edit" + (20 + x), %EventLow%, Kaleidoscope Pro 		; set low weather event
	ControlSetText, % "Edit" + (21 + x), %EventHigh%, Kaleidoscope Pro 		; set high weather event
	ControlSetText, % "Edit" + (22 + x), %WindLow%, Kaleidoscope Pro 		; set low wind speed
	ControlSetText, % "Edit" + (23 + x), %WindHigh%, Kaleidoscope Pro 		; set high wind speed
	ControlSetText, % "Edit" + (24 + x), %CloudLow%, Kaleidoscope Pro 		; set low cloud cover %
	ControlSetText, % "Edit" + (25 + x), %CloudHigh%, Kaleidoscope Pro 		; set high cloud cover %
	ControlSetText, % "Edit" + (26 + x), %kalver%, Kaleidoscope Pro 		; set Kaleidoscope version
	ControlSetText, % "Edit" + (27 + x), %spp_list%, Kaleidoscope Pro 		; set species list
	ControlSetText, % "Edit" + (28 + x), %manual%, Kaleidoscope Pro 		; set manual vetter
	Control, Choose, 1, ComboBox4, Kaleidoscope Pro 						; set Notes output to Append
	Control, Choose, 1, ComboBox5, Kaleidoscope Pro 						; set Access type to Private
	Control, Choose, 1, ComboBox6, Kaleidoscope Pro 						; set License type to Private
	Sleep, 150
	focus_count := 31 + x
	Loop, %focus_count%														; loop through and select each Edit box so Kaleidoscope detects it was entered
	{
		if (A_Index < 4)
			continue
		ControlFocus, % "Edit" + A_Index, Kaleidoscope Pro
		Sleep, 50
	}
	ControlFocus, Edit4, Kaleidoscope Pro									; go back to first metadata field to user sees it
	MsgBox, 0, Kaleidoscope Assistant (SEBatHub) - Ready, The script has finished.`nPlease review the settings and metadata`, then click Process Files.
	;MsgBox, 4, Kaleidoscope Assistant (SEBatHub) - Ready, The script has finished.`nPlease review the settings and click Yes to process files or No to cancel.
	;IfMsgBox Yes
		;ControlClick, Button1, Kaleidoscope Pro 							; click to process files
	;IfMsgBox No
	;	Return
Return

OK_WithinLoop:
	Gui, Submit, Destroy
	return
